@@ -37,6 +37,8 @@
 #include "liblwgeom_internal.h"
 #include "lwgeom_log.h"
 
+#include "R.h"
+
 /* Default allocators */
 static void * default_allocator(size_t size);
 static void default_freeor(void *mem);
@@ -119,7 +121,8 @@ default_noticereporter(const char *fmt, va_list ap)
 	char msg[LW_MSG_MAXLEN+1];
 	vsnprintf (msg, LW_MSG_MAXLEN, fmt, ap);
 	msg[LW_MSG_MAXLEN]='\0';
-	/* fprintf(stderr, "%s\n", msg); */
+	// fprintf(stderr, "%s\n", msg);
+	Rprintf("%s\n", msg);
 }
 
 static void
@@ -134,7 +137,8 @@ default_debuglogger(int level, const char *fmt, va_list ap)
 			msg[i] = ' ';
 		vsnprintf(msg+i, LW_MSG_MAXLEN-i, fmt, ap);
 		msg[LW_MSG_MAXLEN]='\0';
-		/* fprintf(stderr, "%s\n", msg); */
+		// fprintf(stderr, "%s\n", msg);
+		Rprintf("%s\n", msg);
 	}
 }
 
@@ -144,7 +148,9 @@ default_errorreporter(const char *fmt, va_list ap)
 	char msg[LW_MSG_MAXLEN+1];
 	vsnprintf (msg, LW_MSG_MAXLEN, fmt, ap);
 	msg[LW_MSG_MAXLEN]='\0';
-	/* fprintf(stderr, "%s\n", msg); exit(1); */
+	// fprintf(stderr, "%s\n", msg);
+	// exit(1);
+	error("%s\n", msg);
 }
 
 /**
@@ -211,8 +217,6 @@ lwdebug(int level, const char *fmt, ...)
 	va_end(ap);
 }
 
-
-
 const char*
 lwtype_name(uint8_t type)
 {
@@ -245,37 +249,13 @@ lwfree(void *mem)
 	lwfree_var(mem);
 }
 
-/*
- * Removes trailing zeros and dot for a %f formatted number.
- * Modifies input.
- */
-void
-trim_trailing_zeros(char *str)
+char *
+lwstrdup(const char* a)
 {
-	char *ptr, *totrim=NULL;
-	int len;
-	int i;
-
-	LWDEBUGF(3, "input: %s", str);
-
-	ptr = strchr(str, '.');
-	if ( ! ptr ) return; /* no dot, no decimal digits */
-
-	LWDEBUGF(3, "ptr: %s", ptr);
-
-	len = strlen(ptr);
-	for (i=len-1; i; i--)
-	{
-		if ( ptr[i] != '0' ) break;
-		totrim=&ptr[i];
-	}
-	if ( totrim )
-	{
-		if ( ptr == totrim-1 ) *ptr = '\0';
-		else *totrim = '\0';
-	}
-
-	LWDEBUGF(3, "output: %s", str);
+	size_t l = strlen(a)+1;
+	char *b = lwalloc(l);
+	strncpy(b, a, l);
+	return b;
 }
 
 /*
@@ -287,7 +267,7 @@ trim_trailing_zeros(char *str)
  * The caller should specify start or end truncation by setting the truncdirection
  * parameter as follows:
  *    0 - start truncation (i.e. characters are removed from the beginning)
- *    1 - end trunctation (i.e. characters are removed from the end)
+ *    1 - end truncation (i.e. characters are removed from the end)
  */
 
 char *lwmessage_truncate(char *str, int startpos, int endpos, int maxlength, int truncdirection)
@@ -354,29 +334,8 @@ char *lwmessage_truncate(char *str, int startpos, int endpos, int maxlength, int
 	return output;
 }
 
-
-char
-getMachineEndian(void)
-{
-	static int endian_check_int = 1; /* dont modify this!!! */
-
-	return *((char *) &endian_check_int); /* 0 = big endian | xdr,
-	                                       * 1 = little endian | ndr
-	                                       */
-}
-
-
-void
-error_if_srid_mismatch(int srid1, int srid2)
-{
-	if ( srid1 != srid2 )
-	{
-		lwerror("Operation on mixed SRID geometries");
-	}
-}
-
-int
-clamp_srid(int srid)
+int32_t
+clamp_srid(int32_t srid)
 {
 	int newsrid = srid;
 
@@ -396,3 +355,210 @@ clamp_srid(int srid)
 	return newsrid;
 }
 
+
+
+
+/* Structure for the type array */
+struct geomtype_struct
+{
+	char *typename;
+	int type;
+	int z;
+	int m;
+};
+
+/* Type array. Note that the order of this array is important in
+   that any typename in the list must *NOT* occur within an entry
+   before it. Otherwise if we search for "POINT" at the top of the
+   list we would also match MULTIPOINT, for example. */
+
+struct geomtype_struct geomtype_struct_array[] =
+{
+	{ "GEOMETRYCOLLECTIONZM", COLLECTIONTYPE, 1, 1 },
+	{ "GEOMETRYCOLLECTIONZ", COLLECTIONTYPE, 1, 0 },
+	{ "GEOMETRYCOLLECTIONM", COLLECTIONTYPE, 0, 1 },
+	{ "GEOMETRYCOLLECTION", COLLECTIONTYPE, 0, 0 },
+
+	{ "GEOMETRYZM", 0, 1, 1 },
+	{ "GEOMETRYZ", 0, 1, 0 },
+	{ "GEOMETRYM", 0, 0, 1 },
+	{ "GEOMETRY", 0, 0, 0 },
+
+	{ "POLYHEDRALSURFACEZM", POLYHEDRALSURFACETYPE, 1, 1 },
+	{ "POLYHEDRALSURFACEZ", POLYHEDRALSURFACETYPE, 1, 0 },
+	{ "POLYHEDRALSURFACEM", POLYHEDRALSURFACETYPE, 0, 1 },
+	{ "POLYHEDRALSURFACE", POLYHEDRALSURFACETYPE, 0, 0 },
+
+	{ "TINZM", TINTYPE, 1, 1 },
+	{ "TINZ", TINTYPE, 1, 0 },
+	{ "TINM", TINTYPE, 0, 1 },
+	{ "TIN", TINTYPE, 0, 0 },
+
+	{ "CIRCULARSTRINGZM", CIRCSTRINGTYPE, 1, 1 },
+	{ "CIRCULARSTRINGZ", CIRCSTRINGTYPE, 1, 0 },
+	{ "CIRCULARSTRINGM", CIRCSTRINGTYPE, 0, 1 },
+	{ "CIRCULARSTRING", CIRCSTRINGTYPE, 0, 0 },
+
+	{ "COMPOUNDCURVEZM", COMPOUNDTYPE, 1, 1 },
+	{ "COMPOUNDCURVEZ", COMPOUNDTYPE, 1, 0 },
+	{ "COMPOUNDCURVEM", COMPOUNDTYPE, 0, 1 },
+	{ "COMPOUNDCURVE", COMPOUNDTYPE, 0, 0 },
+
+	{ "CURVEPOLYGONZM", CURVEPOLYTYPE, 1, 1 },
+	{ "CURVEPOLYGONZ", CURVEPOLYTYPE, 1, 0 },
+	{ "CURVEPOLYGONM", CURVEPOLYTYPE, 0, 1 },
+	{ "CURVEPOLYGON", CURVEPOLYTYPE, 0, 0 },
+
+	{ "MULTICURVEZM", MULTICURVETYPE, 1, 1 },
+	{ "MULTICURVEZ", MULTICURVETYPE, 1, 0 },
+	{ "MULTICURVEM", MULTICURVETYPE, 0, 1 },
+	{ "MULTICURVE", MULTICURVETYPE, 0, 0 },
+
+	{ "MULTISURFACEZM", MULTISURFACETYPE, 1, 1 },
+	{ "MULTISURFACEZ", MULTISURFACETYPE, 1, 0 },
+	{ "MULTISURFACEM", MULTISURFACETYPE, 0, 1 },
+	{ "MULTISURFACE", MULTISURFACETYPE, 0, 0 },
+
+	{ "MULTILINESTRINGZM", MULTILINETYPE, 1, 1 },
+	{ "MULTILINESTRINGZ", MULTILINETYPE, 1, 0 },
+	{ "MULTILINESTRINGM", MULTILINETYPE, 0, 1 },
+	{ "MULTILINESTRING", MULTILINETYPE, 0, 0 },
+
+	{ "MULTIPOLYGONZM", MULTIPOLYGONTYPE, 1, 1 },
+	{ "MULTIPOLYGONZ", MULTIPOLYGONTYPE, 1, 0 },
+	{ "MULTIPOLYGONM", MULTIPOLYGONTYPE, 0, 1 },
+	{ "MULTIPOLYGON", MULTIPOLYGONTYPE, 0, 0 },
+
+	{ "MULTIPOINTZM", MULTIPOINTTYPE, 1, 1 },
+	{ "MULTIPOINTZ", MULTIPOINTTYPE, 1, 0 },
+	{ "MULTIPOINTM", MULTIPOINTTYPE, 0, 1 },
+	{ "MULTIPOINT", MULTIPOINTTYPE, 0, 0 },
+
+	{ "LINESTRINGZM", LINETYPE, 1, 1 },
+	{ "LINESTRINGZ", LINETYPE, 1, 0 },
+	{ "LINESTRINGM", LINETYPE, 0, 1 },
+	{ "LINESTRING", LINETYPE, 0, 0 },
+
+	{ "TRIANGLEZM", TRIANGLETYPE, 1, 1 },
+	{ "TRIANGLEZ", TRIANGLETYPE, 1, 0 },
+	{ "TRIANGLEM", TRIANGLETYPE, 0, 1 },
+	{ "TRIANGLE", TRIANGLETYPE, 0, 0 },
+
+	{ "POLYGONZM", POLYGONTYPE, 1, 1 },
+	{ "POLYGONZ", POLYGONTYPE, 1, 0 },
+	{ "POLYGONM", POLYGONTYPE, 0, 1 },
+	{ "POLYGON", POLYGONTYPE, 0, 0 },
+
+	{ "POINTZM", POINTTYPE, 1, 1 },
+	{ "POINTZ", POINTTYPE, 1, 0 },
+	{ "POINTM", POINTTYPE, 0, 1 },
+	{ "POINT", POINTTYPE, 0, 0 }
+
+};
+#define GEOMTYPE_STRUCT_ARRAY_LEN (sizeof geomtype_struct_array/sizeof(struct geomtype_struct))
+
+/*
+* We use a very simple upper case mapper here, because the system toupper() function
+* is locale dependent and may have trouble mapping lower case strings to the upper
+* case ones we expect (see, the "Turkisk I", http://www.i18nguy.com/unicode/turkish-i18n.html)
+* We could also count on PgSQL sending us *lower* case inputs, as it seems to do that
+* regardless of the case the user provides for the type arguments.
+*/
+const char dumb_upper_map[128] = "................................................0123456789.......ABCDEFGHIJKLMNOPQRSTUVWXYZ......ABCDEFGHIJKLMNOPQRSTUVWXYZ.....";
+
+static char dumb_toupper(int in)
+{
+	if ( in < 0 || in > 127 )
+		return '.';
+	return dumb_upper_map[in];
+}
+
+lwflags_t lwflags(int hasz, int hasm, int geodetic)
+{
+	lwflags_t flags = 0;
+	if (hasz)
+		FLAGS_SET_Z(flags, 1);
+	if (hasm)
+		FLAGS_SET_M(flags, 1);
+	if (geodetic)
+		FLAGS_SET_GEODETIC(flags, 1);
+	return flags;
+}
+
+/**
+* Calculate type integer and dimensional flags from string input.
+* Case insensitive, and insensitive to spaces at front and back.
+* Type == 0 in the case of the string "GEOMETRY" or "GEOGRAPHY".
+* Return LW_SUCCESS for success.
+*/
+int geometry_type_from_string(const char *str, uint8_t *type, int *z, int *m)
+{
+	char *tmpstr;
+	size_t tmpstartpos, tmpendpos;
+	size_t i;
+
+	assert(str);
+	assert(type);
+	assert(z);
+	assert(m);
+
+	/* Initialize. */
+	*type = 0;
+	*z = 0;
+	*m = 0;
+
+	/* Locate any leading/trailing spaces */
+	tmpstartpos = 0;
+	for (i = 0; i < strlen(str); i++)
+	{
+		if (str[i] != ' ')
+		{
+			tmpstartpos = i;
+			break;
+		}
+	}
+
+	tmpendpos = strlen(str) - 1;
+	for (i = strlen(str) - 1; i != 0; i--)
+	{
+		if (str[i] != ' ')
+		{
+			tmpendpos = i;
+			break;
+		}
+	}
+
+	/* Copy and convert to upper case for comparison */
+	tmpstr = lwalloc(tmpendpos - tmpstartpos + 2);
+	for (i = tmpstartpos; i <= tmpendpos; i++)
+		tmpstr[i - tmpstartpos] = dumb_toupper(str[i]);
+
+	/* Add NULL to terminate */
+	tmpstr[i - tmpstartpos] = '\0';
+
+	/* Now check for the type */
+	for (i = 0; i < GEOMTYPE_STRUCT_ARRAY_LEN; i++)
+	{
+		if (!strcmp(tmpstr, geomtype_struct_array[i].typename))
+		{
+			*type = geomtype_struct_array[i].type;
+			*z = geomtype_struct_array[i].z;
+			*m = geomtype_struct_array[i].m;
+
+			lwfree(tmpstr);
+
+			return LW_SUCCESS;
+		}
+
+	}
+
+	lwfree(tmpstr);
+
+	return LW_FAILURE;
+}
+
+
+
+
+
+
